package com.github.gkutiel.fbi;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ExecutionException;
import java.util.logging.Logger;

import com.github.gkutiel.fbi.Fbi.Args.Post;
import com.github.gkutiel.fbi.obj.User;
import com.google.common.collect.ObjectArrays;
import com.google.gson.Gson;
import com.ning.http.client.AsyncHttpClient;

public class Fbi {
	public static interface Args {
		public static enum Post implements Args {
			// @formatter:off
			MESSAGE("message"), 
			PICTURE("picture"),
			LINK("link"), 
			NAME("name"),
			CAPTION("caption"),
			DESCRIPTION("description"),
			SOURCE("source"),
			PLACE("place"),
			TAGS("tags");
			//@formatter:on

			private final String key;

			Post(final String key) {
				this.key = key;
			}

			@Override
			public String key() {
				return key;
			}
		}

		String key();
	}

	public static class Arguments<T extends Args> {
		private final Map<T, String> args = new HashMap<>();
		public Param[] get() {
			final Param[] params = new Param[args.size()];

			int i = 0;
			for (final Entry<T, String> e : args.entrySet())
				params[i++] = new Param(e.getKey().key(), e.getValue());

			return params;
		}

		public Arguments<T> set(final T key, final String val) {
			args.put(key, val);
			return this;
		}
	}

	public static class Param {
		private final String key;
		private final String val;

		public Param(final String key, final String val) {
			this.key = key;
			this.val = val;
		}

		@Override
		public String toString() {
			return key + "=" + val;
		}
	}

	private static final Logger LOGGER = Logger.getLogger(Fbi.class.getCanonicalName());
	private static final Gson GSON = new Gson();
	private static final AsyncHttpClient HTTP_CLIENT = new AsyncHttpClient();
	private static final Param PARAM_POST = new Param("method", "post");

	public static String get(final String url) {
		try {
			LOGGER.info("-->\t" + url);
			final String response = HTTP_CLIENT.prepareGet(url).execute().get().getResponseBody();
			LOGGER.info("<--\t" + response);
			return response;
		} catch (IOException | InterruptedException | ExecutionException e) {
			throw new RuntimeException(e);
		}
	}

	public static Param p(final String key, final Object val) {
		return new Param(key, val.toString());
	}

	private static void addParam(final StringBuilder url, final Param p) throws UnsupportedEncodingException {
		url.append(p.key);
		url.append("=");
		url.append(URLEncoder.encode(p.val, "UTF-8"));
	}

	private static String url(final String path, final Param... params) {
		try {
			final StringBuilder url = new StringBuilder();
			url.append("https://graph.facebook.com/");
			url.append(path);

			if (params != null && params.length > 0) {
				url.append("?");
				addParam(url, params[0]);
				for (int i = 1; i < params.length; i++) {
					url.append("&");
					addParam(url, params[i]);
				}
			}

			return url.toString();
		} catch (final UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}

	public final String accessToken;

	public final Param accessTokenParam;

	private User me;

	public Fbi(final String accessToken) {
		this.accessToken = accessToken;
		accessTokenParam = new Param("access_token", accessToken);
	}

	public <T> T fql(final Class<T> c, final String query) {
		LOGGER.info("? " + query);
		return GSON.fromJson(fql(query), c);
	}

	public String fql(final String query) {
		return get(url("fql", accessTokenParam, p("q", query), p("format", "json")));
	}

	public <T> T get(final Class<T> c, final String path, final Param... params) {
		return GSON.fromJson(get(path, params), c);
	}

	public String get(final String path, final Param... params) {
		return get(url(path, params));
	}

	public User me() {
		if (me == null) me = get(User.class, "/me", accessTokenParam, p("fields", "id,name,picture"));
		return me;
	}

	public void post(final String path, final Param... params) {
		get(path, ObjectArrays.concat(params, PARAM_POST));
	}

	public void publish(final Arguments<Post> postArgs) {
		post(me().id + "/feed", ObjectArrays.concat(postArgs.get(), accessTokenParam));
	}
}
